mod sort_button ;

use {
	gtk4 :: {
		Box , Image , Label , ListBox , ListBoxRow , Popover ,
		Align :: * , Orientation :: * ,
		prelude :: * ,
	} ,
	libadwaita :: * ,
	std :: { cell :: * , ops :: * } ,
} ;

use crate :: {
	ui :: {
		component :: * , utility :: * ,
		collatable_container :: { * , collation_menu :: sort_button :: * } } ,
	utility :: * ,
} ;



pub struct FilmCollationMenu { widget : Box }
pub struct SeriesCollationMenu { widget : Box }

impl FilmCollationMenu {
	pub fn new < F : Fn ( FilmsSortedBy , SortingDirection ) + 'static > ( on_sort : F ) -> Self {
		let film_sort_button = FilmSortButton :: new (on_sort) ;

		let widget = g_box ! (
			@ orientation : Horizontal ,
			@ halign : Center ,
			@ spacing : 20 ,
			@ widget_name : "film-collation-menu" ,
			@ css_classes : [ "toolbar" ] ,
			* film_sort_button . get_widget ()
		) ;

		Self { widget }
	}
}
impl SeriesCollationMenu {
	pub fn new < F : Fn (SeriesSortedBy) + 'static > ( on_sort : F ) -> Self {
		let widget = Box :: builder ()
			. orientation (Horizontal)
			. halign (Center)
			. spacing (20)
			. css_classes ( ["toolbar"] )
			. build () ;

		widget . append ( & create_sort_button ( & create_series_sort_menu (on_sort) ) ) ;
		widget . append ( & SplitButton :: builder () . label ("Filter") . build () ) ;

		Self { widget }
	}
}

impl Component <Box> for FilmCollationMenu {
	fn get_widget ( & self ) -> & Box { & self . widget }
}
impl Component <Box> for SeriesCollationMenu {
	fn get_widget ( & self ) -> & Box { & self . widget }
}

fn create_sort_button ( sort_menu : & Popover ) -> SplitButton {
	SplitButton :: builder ()
		. child ( & label ! ("Sort") )
		. popover (sort_menu)
		. build ()
}

fn create_films_sort_menu < F : Fn ( FilmsSortedBy , SortingDirection ) + 'static > (
	sorting : & 'static RefCell < ( FilmsSortedBy , SortingDirection ) > ,
	on_sort : F ,
) -> Popover {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "Release date" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "Runtime" , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | {
		let sorting_property = match row . index () {
			0 => FilmsSortedBy :: Name ,
			1 => FilmsSortedBy :: ReleaseDate ,
			2 => FilmsSortedBy :: Runtime ,
			_ => panic ! () ,
		} ;
		on_sort ( sorting_property , SortingDirection :: Ascending )
	} ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}
fn create_series_sort_menu < F : Fn (SeriesSortedBy) + 'static > ( on_sort : F ) -> Popover {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false ) ) ;
	container . append ( & create_sort_menu_entry ( "First release date" , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | on_sort ( match row . index () {
		0 => SeriesSortedBy :: Name ,
		1 => SeriesSortedBy :: FirstReleaseDate ,
		_ => panic ! () ,
	} ) ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}

fn create_sort_menu_entry ( label : & str , reverse : bool ) -> ListBoxRow {
	let icon = match reverse {
		false => Image :: from_icon_name ("view-sort-ascending-symbolic") ,
		true => Image :: from_icon_name ("view-sort-descending-symbolic") ,
	} ;

	let container = g_box ! (
		@ orientation : Horizontal ,
		Label :: builder ()
			. label (label)
			. hexpand (true)
			. halign (Start)
			. build () ,
		icon ,
	) ;
	container . set_spacing (20) ;

	ListBoxRow :: builder () . child ( & container ) . build ()
}

fn create_filter_button () -> SplitButton {
	SplitButton :: builder () . label ("Filter") . build ()
}
