use { gtk4 :: Align :: * , libadwaita :: * , std :: cell :: * } ;

use crate :: { utility :: * , ui :: { * , utility :: * } } ;



pub struct FilmSortButton {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell < ( FilmsSortedBy , SortingDirection ) > ,
}

impl FilmSortButton {
	pub fn new < F : Fn ( FilmsSortedBy , SortingDirection ) + 'static > ( on_sort : F ) -> Self {
		let list_box = list_box ! (
			g_box ! (
				@ orientation : Horizontal , @ spacing : 20 ,
				label ! ( @ hexpand : true , @ halign : Start , "Name" ) ,
				icon ! ("view-sort-ascending-symbolic") ,
			) ,
			g_box ! (
				@ orientation : Horizontal , @ spacing : 20 ,
				label ! ( @ hexpand : true , @ halign : Start , "Release date" ) ,
				icon ! ("view-sort-ascending-symbolic") ,
			) ,
			g_box ! (
				@ orientation : Horizontal , @ spacing : 20 ,
				label ! ( @ hexpand : true , @ halign : Start , "Runtime" ) ,
				icon ! ("view-sort-ascending-symbolic") ,
			) ,
		) ;

		let widget = split_button ! (
			@ popover : popover ! ( @ css_classes : [ "menu" ] , list_box ) ,
			label ! ("Sort") ,
		) ;

		let previous_sorting = leak ( RefCell :: new (
			( FilmsSortedBy :: Name , SortingDirection :: Ascending )
		) ) ;

		list_box . connect_row_activated ( move | _ , row | {
			let sorting_property = match row . index () {
				0 => FilmsSortedBy :: Name ,
				1 => FilmsSortedBy :: ReleaseDate ,
				2 => FilmsSortedBy :: Runtime ,
				_ => panic ! () ,
			} ;

			let previous_sorting_property = previous_sorting . borrow () . 0 ;
			if sorting_property == previous_sorting_property {
				let previous_sorting_direction = previous_sorting . borrow () . 1 ;
				match previous_sorting_direction {
					SortingDirection :: Ascending => {
						previous_sorting . replace ( ( sorting_property , SortingDirection :: Descending ) ) ;
						on_sort ( sorting_property , SortingDirection :: Descending ) ;
					} ,
					SortingDirection :: Descending => {
						previous_sorting . replace ( ( sorting_property , SortingDirection :: Ascending ) ) ;
						on_sort ( sorting_property , SortingDirection :: Ascending ) ;
					} ,
				}
			} else {
				previous_sorting . replace ( ( sorting_property , SortingDirection :: Ascending ) ) ;
				on_sort ( sorting_property , SortingDirection :: Ascending ) ;
			}
		} ) ;

		Self { widget , previous_sorting }
	}
}

impl Component <SplitButton> for FilmSortButton {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}
